@@ -533,14 +533,14 @@ add_config_file (GoaDaemon     *self,
                   if (provider_type != NULL)
                     provider = goa_provider_get_for_provider_type (provider_type);
 
-                  needs_update = g_key_file_remove_group (key_file, groups[n], NULL) || needs_update;
-
                   if (provider == NULL)
                     {
-                      g_debug ("Unsupported account type %s for ID %s (no provider)", provider_type, id);
+                      g_warning ("Unsupported account type %s for ID %s (no provider)", provider_type, id);
                       goto cleanup_and_continue;
                     }
 
+                  needs_update = g_key_file_remove_group (key_file, groups[n], NULL);
+
                   error = NULL;
                   if (!goa_utils_delete_credentials_for_id_sync (provider, id, NULL, &error))
                     {
@@ -560,7 +560,7 @@ add_config_file (GoaDaemon     *self,
             }
           else
             {
-              needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL) || needs_update;
+              needs_update = g_key_file_remove_key (key_file, groups[n], "SessionId", NULL);
             }
 
           g_hash_table_insert (group_name_to_key_file_data,
@@ -645,7 +645,7 @@ update_account_object (GoaDaemon           *self,
   provider = goa_provider_get_for_provider_type (type);
   if (provider == NULL)
     {
-      g_debug ("Unsupported account type %s for identity %s (no provider)", type, identity);
+      g_warning ("Unsupported account type %s for identity %s (no provider)", type, identity);
       goto out;
     }
 
@@ -726,6 +726,7 @@ process_config_entries (GoaDaemon  *self,
           continue;
 
         id = account_group_to_id (group);
+        g_assert (id != NULL);
 
         /* create and validate object path */
         object_path = g_strdup_printf ("/org/gnome/OnlineAccounts/Accounts/%s", id);
