@@ -87,6 +87,8 @@ build_object (GoaProvider         *provider,
   GoaExchange *exchange = NULL;
   GoaMail *mail = NULL;
   GoaPasswordBased *password_based = NULL;
+  GKeyFile *goa_conf;
+  const gchar *provider_type;
   gboolean calendar_enabled;
   gboolean contacts_enabled;
   gboolean mail_enabled;
@@ -116,11 +118,14 @@ build_object (GoaProvider         *provider,
                         NULL);
     }
 
+  provider_type = goa_provider_get_provider_type (provider);
+  goa_conf = goa_util_open_goa_conf ();
   account = goa_object_get_account (GOA_OBJECT (object));
 
   /* Email */
   mail = goa_object_get_mail (GOA_OBJECT (object));
-  mail_enabled = g_key_file_get_boolean (key_file, group, "MailEnabled", NULL);
+  mail_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_MAIL) &&
+                 g_key_file_get_boolean (key_file, group, "MailEnabled", NULL);
   if (mail_enabled)
     {
       if (mail == NULL)
@@ -140,13 +145,17 @@ build_object (GoaProvider         *provider,
     }
 
   /* Calendar */
-  calendar_enabled = g_key_file_get_boolean (key_file, group, "CalendarEnabled", NULL);
+  calendar_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_CALENDAR) &&
+                     g_key_file_get_boolean (key_file, group, "CalendarEnabled", NULL);
   goa_object_skeleton_attach_calendar (object, NULL, calendar_enabled, FALSE);
 
   /* Contacts */
-  contacts_enabled = g_key_file_get_boolean (key_file, group, "ContactsEnabled", NULL);
+  contacts_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_CONTACTS) &&
+                     g_key_file_get_boolean (key_file, group, "ContactsEnabled", NULL);
   goa_object_skeleton_attach_contacts (object, NULL, contacts_enabled, FALSE);
 
+  g_clear_pointer (&goa_conf, g_key_file_free);
+
   /* Exchange */
   exchange = goa_object_get_exchange (GOA_OBJECT (object));
   if (exchange == NULL)
@@ -305,10 +314,6 @@ on_username_or_server_changed (GtkEditable    *editable,
   const char *server = NULL;
   const char *username = NULL;
 
-  /* Reset the preference to ignore SSL/TLS errors
-   */
-  data->accept_ssl_errors = FALSE;
-
   email = gtk_editable_get_text (GTK_EDITABLE (data->email_address));
   if (!goa_utils_parse_email_address (email, NULL, NULL))
     goto out;
@@ -333,10 +338,6 @@ on_email_or_password_changed (GtkEditable    *editable,
   g_autofree char *email_localpart = NULL;
   g_autofree char *email_domain = NULL;
 
-  /* Reset the preference to ignore SSL/TLS errors
-   */
-  data->accept_ssl_errors = FALSE;
-
   email = gtk_editable_get_text (GTK_EDITABLE (data->email_address));
   if (!goa_utils_parse_email_address (email, &email_localpart, &email_domain))
     goto out;
@@ -386,6 +387,24 @@ create_setup_page (GoaProvider    *provider,
   data->password = goa_provider_dialog_add_password_entry (dialog, group, _("_Password"));
   goa_provider_dialog_add_description (dialog, NULL, _("Exchange account details will be auto-detected from your email address when possible"));
 
+  if (new_account)
+    {
+      group = goa_provider_dialog_add_group (dialog, _("Account Details"));
+      data->username = goa_provider_dialog_add_entry (dialog, group, _("User_name"));
+      data->server = goa_provider_dialog_add_entry (dialog, group, _("_Domain"));
+      goa_provider_dialog_add_description (dialog, data->server, _("Example domain: example.com"));
+
+      g_signal_connect (data->username,
+                        "changed",
+                        G_CALLBACK (on_username_or_server_changed),
+                        data);
+      g_signal_connect (data->server,
+                        "changed",
+                        G_CALLBACK (on_username_or_server_changed),
+                        data);
+    }
+
+  gtk_widget_grab_focus ((new_account) ? data->email_address : data->password);
   g_signal_connect (data->email_address,
                     "changed",
                     G_CALLBACK (on_email_or_password_changed),
@@ -394,22 +413,6 @@ create_setup_page (GoaProvider    *provider,
                     "changed",
                     G_CALLBACK (on_email_or_password_changed),
                     data);
-
-  group = goa_provider_dialog_add_group (dialog, _("Account Details"));
-  data->username = goa_provider_dialog_add_entry (dialog, group, _("User_name"));
-  data->server = goa_provider_dialog_add_entry (dialog, group, _("_Domain"));
-  goa_provider_dialog_add_description (dialog, data->server, _("Example domain: example.com"));
-
-  g_signal_connect (data->username,
-                    "changed",
-                    G_CALLBACK (on_username_or_server_changed),
-                    data);
-  g_signal_connect (data->server,
-                    "changed",
-                    G_CALLBACK (on_username_or_server_changed),
-                    data);
-
-  gtk_widget_grab_focus ((new_account) ? data->email_address : data->password);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -455,7 +458,6 @@ add_account_autodiscover_cb (GoaEwsClient *client,
 
   if (!goa_ews_client_autodiscover_finish (client, res, &error))
     {
-      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
@@ -542,7 +544,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
 static void
 add_account (GoaProvider         *provider,
              GoaClient           *client,
-             GtkWindow           *parent,
+             GtkWidget           *parent,
              GCancellable        *cancellable,
              GAsyncReadyCallback  callback,
              gpointer             user_data)
@@ -680,7 +682,7 @@ static void
 refresh_account (GoaProvider         *provider,
                  GoaClient           *client,
                  GoaObject           *object,
-                 GtkWindow           *parent,
+                 GtkWidget           *parent,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback,
                  gpointer             user_data)
@@ -691,7 +693,7 @@ refresh_account (GoaProvider         *provider,
   g_assert (GOA_IS_EXCHANGE_PROVIDER (provider));
   g_assert (GOA_IS_CLIENT (client));
   g_assert (GOA_IS_OBJECT (object));
-  g_assert (parent == NULL || GTK_IS_WINDOW (parent));
+  g_assert (parent == NULL || GTK_IS_WIDGET (parent));
   g_assert (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
   data = g_new0 (AddAccountData, 1);
