@@ -173,6 +173,8 @@ build_object (GoaProvider         *provider,
 {
   GoaAccount   *account;
   GoaTicketing *ticketing = NULL;
+  GKeyFile     *goa_conf;
+  const gchar  *provider_type;
   gboolean      ticketing_enabled;
   gboolean      ret = FALSE;
 
@@ -185,10 +187,15 @@ build_object (GoaProvider         *provider,
                                                                               error))
     goto out;
 
+  provider_type = goa_provider_get_provider_type (provider);
+  goa_conf = goa_util_open_goa_conf ();
   account = goa_object_get_account (GOA_OBJECT (object));
 
   ticketing = goa_object_get_ticketing (GOA_OBJECT (object));
-  ticketing_enabled = g_key_file_get_boolean (key_file, group, "TicketingEnabled", NULL);
+  ticketing_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_TICKETING) &&
+                      g_key_file_get_boolean (key_file, group, "TicketingEnabled", NULL);
+
+  g_clear_pointer (&goa_conf, g_key_file_free);
 
   if (ticketing_enabled)
     {
@@ -263,7 +270,7 @@ static void
 refresh_account (GoaProvider         *provider,
                  GoaClient           *client,
                  GoaObject           *object,
-                 GtkWindow           *parent,
+                 GtkWidget           *parent,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback,
                  gpointer             user_data)
@@ -274,7 +281,7 @@ refresh_account (GoaProvider         *provider,
   g_assert (GOA_IS_KERBEROS_PROVIDER (provider));
   g_assert (GOA_IS_CLIENT (client));
   g_assert (GOA_IS_OBJECT (object));
-  g_assert (parent == NULL || GTK_IS_WINDOW (parent));
+  g_assert (parent == NULL || GTK_IS_WIDGET (parent));
   g_assert (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
   task = g_task_new (provider, cancellable, callback, user_data);
@@ -595,7 +602,7 @@ create_account_details_ui (GoaProvider    *self,
   data->principal = goa_provider_dialog_add_entry (dialog, group, _("_Principal"));
   goa_provider_dialog_add_description (dialog, data->principal, _("Example principal: user@EXAMPLE.COM"));
 
-  button = gtk_window_get_default_widget (GTK_WINDOW (dialog));
+  button = adw_dialog_get_default_widget (ADW_DIALOG (dialog));
   gtk_button_set_label (GTK_BUTTON (button), _("_Sign in…"));
 
   gtk_widget_grab_focus (data->principal);
@@ -754,7 +761,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
 static void
 add_account (GoaProvider         *provider,
              GoaClient           *client,
-             GtkWindow           *parent,
+             GtkWidget           *parent,
              GCancellable        *cancellable,
              GAsyncReadyCallback  callback,
              gpointer             user_data)
@@ -872,26 +879,11 @@ ensure_credentials_sync (GoaProvider    *provider,
       if (!ticket_synced)
         {
           g_dbus_error_strip_remote_error (lookup_error);
-          if (lookup_error->domain == GOA_IDENTITY_MANAGER_ERROR)
-            {
-              lookup_error->domain = GOA_ERROR;
-
-              switch ((GoaIdentityManagerError)lookup_error->code)
-                {
-                case GOA_IDENTITY_MANAGER_ERROR_INITIALIZING:
-                  lookup_error->code = GOA_ERROR_FAILED;
-                  break;
-
-                case GOA_IDENTITY_MANAGER_ERROR_IDENTITY_NOT_FOUND:
-                case GOA_IDENTITY_MANAGER_ERROR_CREATING_IDENTITY:
-                case GOA_IDENTITY_MANAGER_ERROR_ACCESSING_CREDENTIALS:
-                case GOA_IDENTITY_MANAGER_ERROR_UNSUPPORTED_CREDENTIALS:
-                  lookup_error->code = GOA_ERROR_NOT_AUTHORIZED;
-                  break;
-                }
-            }
-
-          g_propagate_error (error, g_steal_pointer (&lookup_error));
+          g_set_error_literal (error,
+                               GOA_ERROR,
+                               GOA_ERROR_NOT_AUTHORIZED,
+                               lookup_error->message);
+          g_error_free (lookup_error);
           goto out;
         }
 
