@@ -259,6 +259,8 @@ build_object (GoaProvider        *provider,
 {
   GoaMsGraphProvider *self = GOA_MS_GRAPH_PROVIDER (provider);
   GoaAccount *account = NULL;
+  GKeyFile *goa_conf;
+  const gchar  *provider_type;
   const gchar *identity = NULL;
   gboolean files_enabled = FALSE;
   gchar *uri_onedrive = NULL;
@@ -273,14 +275,18 @@ build_object (GoaProvider        *provider,
                                                                               error))
     goto out;
 
+  provider_type = goa_provider_get_provider_type (provider);
+  goa_conf = goa_util_open_goa_conf ();
   account = goa_object_get_account (GOA_OBJECT (object));
   identity = goa_account_get_identity (account);
 
   /* Files */
-  files_enabled = g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
+  files_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_FILES) &&
+                  g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
   uri_onedrive = g_strconcat ("onedrive://", identity, "/", NULL);
   goa_object_skeleton_attach_files (object, uri_onedrive, files_enabled, FALSE);
   g_free (uri_onedrive);
+  g_clear_pointer (&goa_conf, g_key_file_free);
 
   self->client_id = g_key_file_get_string (key_file, group, "client_id", NULL);
   self->redirect_uri = g_key_file_get_string (key_file, group, "redirect_uri", NULL);
@@ -384,7 +390,7 @@ create_account_details_ui (GoaProvider *provider,
       data->tenant_id_entry = goa_provider_dialog_add_entry (dialog, group, _("_Tenant ID"));
       goa_provider_dialog_add_description (dialog, data->tenant_id_entry, _("Example ID: 00000000-0000-0000-0000-000000000000"));
 
-      button = gtk_window_get_default_widget (GTK_WINDOW (dialog));
+      button = adw_dialog_get_default_widget (ADW_DIALOG (dialog));
       gtk_button_set_label (GTK_BUTTON (button), _("_Sign in…"));
 
       g_signal_connect (data->client_id_entry,
@@ -456,7 +462,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
   /* With the provider configured for the client, we chain-up to authorize */
   GOA_PROVIDER_CLASS (goa_ms_graph_provider_parent_class)->add_account (provider,
                                                                         data->client,
-                                                                        GTK_WINDOW (data->dialog),
+                                                                        GTK_WIDGET (data->dialog),
                                                                         cancellable,
                                                                         (GAsyncReadyCallback)add_account_parent_cb,
                                                                         g_object_ref (task));
@@ -465,7 +471,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
 static void
 add_account (GoaProvider         *provider,
              GoaClient           *client,
-             GtkWindow           *parent,
+             GtkWidget           *parent,
              GCancellable        *cancellable,
              GAsyncReadyCallback  callback,
              gpointer             user_data)
