@@ -955,7 +955,7 @@ create_account_details_ui (GoaProvider *provider,
   goa_provider_dialog_push_content (dialog, NULL, content);
 
   /* Set the default widget after it's a child of the window */
-  gtk_window_set_default_widget (GTK_WINDOW (dialog), button);
+  adw_dialog_set_default_widget (ADW_DIALOG (dialog), button);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1020,15 +1020,6 @@ oauth2_secret_collection_for_alias_cb (GObject      *object,
   data->session = secret_collection_for_alias_finish (result, &error);
   if (data->session == NULL)
     {
-      if (error == NULL)
-        {
-          g_warning ("%s(): Failed to get session keyring", G_STRFUNC);
-          g_set_error_literal (&error,
-                               GOA_ERROR,
-                               GOA_ERROR_FAILED,
-                               _("Unknown error"));
-        }
-
       goa_provider_task_return_error (task, g_steal_pointer (&error));
       return;
     }
@@ -1224,7 +1215,7 @@ add_account_secret_cb (SecretCollection *collection,
 static void
 goa_oauth2_provider_add_account (GoaProvider         *provider,
                                  GoaClient           *client,
-                                 GtkWindow           *parent,
+                                 GtkWidget           *parent,
                                  GCancellable        *cancellable,
                                  GAsyncReadyCallback  callback,
                                  gpointer             user_data)
@@ -1327,7 +1318,7 @@ static void
 goa_oauth2_provider_refresh_account (GoaProvider         *provider,
                                      GoaClient           *client,
                                      GoaObject           *object,
-                                     GtkWindow           *parent,
+                                     GtkWidget           *parent,
                                      GCancellable        *cancellable,
                                      GAsyncReadyCallback  callback,
                                      gpointer             user_data)
@@ -1338,7 +1329,7 @@ goa_oauth2_provider_refresh_account (GoaProvider         *provider,
   g_assert (GOA_IS_OAUTH2_PROVIDER (provider));
   g_assert (GOA_IS_CLIENT (client));
   g_assert (GOA_IS_OBJECT (object));
-  g_assert (parent == NULL || GTK_IS_WINDOW (parent));
+  g_assert (parent == NULL || GTK_IS_WIDGET (parent));
   g_assert (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
   data = g_new0 (AccountData, 1);
