@@ -24,6 +24,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "goadavclient.h"
+#include "goadavconfig.h"
 #include "goaprovider.h"
 #include "goaproviderdialog.h"
 #include "goawebdavprovider.h"
@@ -242,6 +243,8 @@ build_object (GoaProvider         *provider,
 {
   GoaAccount *account = NULL;
   g_autoptr (GoaPasswordBased) password_based = NULL;
+  g_autoptr (GKeyFile) goa_conf = NULL;
+  const gchar *provider_type;
   g_autofree char *uri_encoded = NULL;
   g_autofree char *uri_caldav = NULL;
   g_autofree char *uri_carddav = NULL;
@@ -276,26 +279,31 @@ build_object (GoaProvider         *provider,
                         NULL);
     }
 
+  provider_type = goa_provider_get_provider_type (provider);
+  goa_conf = goa_util_open_goa_conf ();
   account = goa_object_get_account (GOA_OBJECT (object));
   identity = goa_account_get_identity (account);
   accept_ssl_errors = g_key_file_get_boolean (key_file, group, "AcceptSslErrors", NULL);
 
   /* Calendar */
-  calendar_enabled = g_key_file_get_boolean (key_file, group, "CalendarEnabled", NULL);
+  calendar_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_CALENDAR) &&
+                     g_key_file_get_boolean (key_file, group, "CalendarEnabled", NULL);
   uri_caldav = g_key_file_get_string (key_file, group, "CalDavUri", NULL);
   uri_encoded = uri_encode_identity (uri_caldav, identity, FALSE);
   goa_object_skeleton_attach_calendar (object, uri_encoded, calendar_enabled, accept_ssl_errors);
   g_clear_pointer (&uri_encoded, g_free);
 
   /* Contacts */
-  contacts_enabled = g_key_file_get_boolean (key_file, group, "ContactsEnabled", NULL);
+  contacts_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_CONTACTS) &&
+                     g_key_file_get_boolean (key_file, group, "ContactsEnabled", NULL);
   uri_carddav = g_key_file_get_string (key_file, group, "CardDavUri", NULL);
   uri_encoded = uri_encode_identity (uri_carddav, identity, FALSE);
   goa_object_skeleton_attach_contacts (object, uri_encoded, contacts_enabled, accept_ssl_errors);
   g_clear_pointer (&uri_encoded, g_free);
 
   /* Files */
-  files_enabled = g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
+  files_enabled = goa_util_provider_feature_is_enabled (goa_conf, provider_type, GOA_PROVIDER_FEATURE_FILES) &&
+                  g_key_file_get_boolean (key_file, group, "FilesEnabled", NULL);
   uri_vfs = g_key_file_get_string (key_file, group, "Uri", NULL);
   uri_encoded = uri_encode_identity (uri_vfs, identity, TRUE);
   goa_object_skeleton_attach_files (object, uri_encoded, files_enabled, accept_ssl_errors);
@@ -328,12 +336,12 @@ build_object (GoaProvider         *provider,
 
 static struct
 {
-  GoaProviderFeatures feature;
+  const char *service;
   const char *key;
 } keyfile_endpoints[] = {
-  { GOA_PROVIDER_FEATURE_CALENDAR, "CalDavUri" },
-  { GOA_PROVIDER_FEATURE_CONTACTS, "CardDavUri" },
-  { GOA_PROVIDER_FEATURE_FILES, "Uri" },
+  { GOA_SERVICE_TYPE_CALDAV, "CalDavUri" },
+  { GOA_SERVICE_TYPE_CARDDAV, "CardDavUri" },
+  { GOA_SERVICE_TYPE_WEBDAV, "Uri" },
 };
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -373,9 +381,11 @@ ensure_credentials_sync (GoaProvider         *provider,
       uri = goa_util_lookup_keyfile_string (object, keyfile_endpoints[i].key);
       if (uri != NULL && *uri != '\0')
         {
+          g_autoptr(GoaDavConfig) config = NULL;
+
+          config = goa_dav_config_new (keyfile_endpoints[i].service, uri, username);
           ret = goa_dav_client_check_sync (dav_client,
-                                           uri,
-                                           username,
+                                           config,
                                            password,
                                            accept_ssl_errors,
                                            cancellable,
@@ -426,9 +436,11 @@ typedef struct
   GoaObject *object;
   gboolean accept_ssl_errors;
 
-  GoaDavConfiguration *config;
+  GoaDavConfig *caldav;
+  GoaDavConfig *carddav;
+  GoaDavConfig *webdav;
+  GoaDavConfig *check_config;
   GoaProviderFeatures check_stage;
-  char *check_uri;
   char *presentation_identity;
   gboolean is_template;
 
@@ -447,8 +459,10 @@ add_account_data_free (gpointer user_data)
 
   g_clear_object (&data->client);
   g_clear_object (&data->object);
-  g_clear_pointer (&data->config, goa_dav_configuration_free);
-  g_clear_pointer (&data->check_uri, g_free);
+  g_clear_object (&data->caldav);
+  g_clear_object (&data->carddav);
+  g_clear_object (&data->webdav);
+  g_clear_object (&data->check_config);
   g_clear_pointer (&data->presentation_identity, g_free);
   g_free (data);
 }
@@ -463,10 +477,6 @@ on_uri_username_or_password_changed (GtkEditable *editable, gpointer user_data)
   const char *address;
   g_autofree char *uri = NULL;
 
-  /* Reset the preference to ignore SSL/TLS errors
-   */
-  data->accept_ssl_errors = FALSE;
-
   address = gtk_editable_get_text (GTK_EDITABLE (data->uri));
   uri = dav_normalize_uri (address, NULL, NULL);
 
@@ -532,7 +542,10 @@ create_account_details_ui (GoaProvider    *provider,
         {
           uri = goa_util_lookup_keyfile_string (data->object, keyfile_endpoints[i].key);
           if (uri != NULL && *uri != '\0')
-            break;
+            {
+              data->check_config = goa_dav_config_new (keyfile_endpoints[i].service, uri, username);
+              break;
+            }
 
           g_clear_pointer (&uri, g_free);
         }
@@ -605,12 +618,12 @@ add_account_credentials (GTask *task)
   g_variant_builder_add (&credentials, "{sv}", "password", g_variant_new_string (password));
 
   g_variant_builder_init (&details, G_VARIANT_TYPE ("a{ss}"));
-  g_variant_builder_add (&details, "{ss}", "Uri", data->config->webdav_uri ? data->config->webdav_uri : "");
-  g_variant_builder_add (&details, "{ss}", "CalendarEnabled", data->config->caldav_uri ? "true" : "false");
-  g_variant_builder_add (&details, "{ss}", "CalDavUri", data->config->caldav_uri ? data->config->caldav_uri : "");
-  g_variant_builder_add (&details, "{ss}", "ContactsEnabled", data->config->carddav_uri ? "true" : "false");
-  g_variant_builder_add (&details, "{ss}", "CardDavUri", data->config->carddav_uri ? data->config->carddav_uri : "");
-  g_variant_builder_add (&details, "{ss}", "FilesEnabled", data->config->webdav_uri ? "true" : "false");
+  g_variant_builder_add (&details, "{ss}", "Uri", data->webdav ? goa_dav_config_get_uri (data->webdav) : "");
+  g_variant_builder_add (&details, "{ss}", "CalendarEnabled", data->caldav ? "true" : "false");
+  g_variant_builder_add (&details, "{ss}", "CalDavUri", data->caldav ? goa_dav_config_get_uri (data->caldav) : "");
+  g_variant_builder_add (&details, "{ss}", "ContactsEnabled", data->carddav ? "true" : "false");
+  g_variant_builder_add (&details, "{ss}", "CardDavUri", data->carddav ? goa_dav_config_get_uri (data->carddav) : "");
+  g_variant_builder_add (&details, "{ss}", "FilesEnabled", data->webdav ? "true" : "false");
   g_variant_builder_add (&details, "{ss}", "AcceptSslErrors", (data->accept_ssl_errors) ? "true" : "false");
 
   goa_manager_call_add_account (goa_client_get_manager (data->client),
@@ -624,17 +637,35 @@ add_account_credentials (GTask *task)
                                 g_object_ref (task));
 }
 
+static gpointer
+services_find_preferred (GPtrArray  *services,
+                         const char *name)
+{
+  for (unsigned int i = 0; i < services->len; i++)
+    {
+      GoaServiceConfig *config = g_ptr_array_index (services, i);
+
+      if (g_str_equal (goa_service_config_get_service (config), name))
+        return g_object_ref (config);
+    }
+
+  return NULL;
+}
+
 static gboolean
-add_account_handle_response (GTask               *task,
-                             GoaDavConfiguration *config)
+add_account_handle_response (GTask     *task,
+                             GPtrArray *services)
 {
   AddAccountData *data = g_task_get_task_data (task);
+  const char *username = NULL;
   const char *base_uri = NULL;
   const char *check_uri = NULL;
+  const char *service = NULL;
+  g_autofree char *normalized_uri = NULL;
   g_autoptr(GError) error = NULL;
 
+  username = gtk_editable_get_text (GTK_EDITABLE (data->username));
   base_uri = gtk_editable_get_text (GTK_EDITABLE (data->uri));
-  g_clear_pointer (&data->check_uri, g_free);
 
   while (TRUE)
     {
@@ -643,13 +674,18 @@ add_account_handle_response (GTask               *task,
         /* This is the primary discovery stage.
          */
         case GOA_PROVIDER_FEATURE_INVALID:
-          if (config != NULL)
+          if (services != NULL)
             {
-              g_clear_pointer (&data->config, goa_dav_configuration_free);
-              data->config = g_steal_pointer (&config);
+              g_clear_object (&data->webdav);
+              data->webdav = services_find_preferred (services, GOA_SERVICE_TYPE_WEBDAV);
+              g_clear_object (&data->caldav);
+              data->caldav = services_find_preferred (services, GOA_SERVICE_TYPE_CALDAV);
+              g_clear_object (&data->carddav);
+              data->carddav = services_find_preferred (services, GOA_SERVICE_TYPE_CARDDAV);
             }
 
           data->check_stage = GOA_PROVIDER_FEATURE_FILES;
+          service = GOA_SERVICE_TYPE_WEBDAV;
           check_uri = gtk_editable_get_text (GTK_EDITABLE (data->webdav_uri));
           break;
 
@@ -657,13 +693,14 @@ add_account_handle_response (GTask               *task,
          * override previously discovered CalDAV/CardDAV endpoints.
          */
         case GOA_PROVIDER_FEATURE_FILES:
-          if (config != NULL)
+          if (services != NULL)
             {
-              g_set_str (&data->config->webdav_uri, config->webdav_uri);
-              g_clear_pointer (&config, goa_dav_configuration_free);
+              g_clear_object (&data->webdav);
+              data->webdav = services_find_preferred (services, GOA_SERVICE_TYPE_WEBDAV);
             }
 
           data->check_stage = GOA_PROVIDER_FEATURE_CALENDAR;
+          service = GOA_SERVICE_TYPE_CALDAV;
           check_uri = gtk_editable_get_text (GTK_EDITABLE (data->caldav_uri));
           break;
 
@@ -671,21 +708,22 @@ add_account_handle_response (GTask               *task,
          * if the server doesn't claim support.
          */
         case GOA_PROVIDER_FEATURE_CALENDAR:
-          if (data->check_uri != NULL)
+          if (data->check_config != NULL)
             {
-              g_free (data->config->caldav_uri);
-              data->config->caldav_uri = g_steal_pointer (&data->check_uri);
+              g_clear_object (&data->caldav);
+              data->caldav = g_steal_pointer (&data->check_config);
             }
 
           data->check_stage = GOA_PROVIDER_FEATURE_CONTACTS;
+          service = GOA_SERVICE_TYPE_CARDDAV;
           check_uri = gtk_editable_get_text (GTK_EDITABLE (data->carddav_uri));
           break;
 
         case GOA_PROVIDER_FEATURE_CONTACTS:
-          if (data->check_uri != NULL)
+          if (data->check_config != NULL)
             {
-              g_free (data->config->carddav_uri);
-              data->config->carddav_uri = g_steal_pointer (&data->check_uri);
+              g_clear_object (&data->carddav);
+              data->carddav = g_steal_pointer (&data->check_config);
             }
 
           /* Set the next stage to default to add_account_credentials() */
@@ -703,8 +741,8 @@ add_account_handle_response (GTask               *task,
     }
 
   /* If the user entered a bunk URI, they probably want to be notified */
-  data->check_uri = dav_normalize_uri (base_uri, check_uri, NULL);
-  if (data->check_uri == NULL)
+  normalized_uri = dav_normalize_uri (base_uri, check_uri, NULL);
+  if (normalized_uri == NULL)
     {
       g_set_error (&error,
                    GOA_ERROR,
@@ -715,6 +753,8 @@ add_account_handle_response (GTask               *task,
       return FALSE;
     }
 
+  data->check_config = goa_dav_config_new (service, normalized_uri, username);
+
   return TRUE;
 }
 
@@ -730,22 +770,17 @@ add_account_check_cb (GoaDavClient *client,
 
   if (!goa_dav_client_check_finish (client, result, &error))
     {
-      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
 
   if (add_account_handle_response (task, NULL))
     {
-      const char *username = NULL;
       const char *password = NULL;
 
-      username = gtk_editable_get_text (GTK_EDITABLE (data->username));
       password = gtk_editable_get_text (GTK_EDITABLE (data->password));
-
       goa_dav_client_check (client,
-                            data->check_uri,
-                            username,
+                            data->check_config,
                             password,
                             data->accept_ssl_errors,
                             cancellable,
@@ -762,31 +797,28 @@ add_account_discover_cb (GoaDavClient *client,
   g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
   AddAccountData *data = g_task_get_task_data (task);
   GCancellable *cancellable = g_task_get_cancellable (task);
-  GoaDavConfiguration *config = NULL;
-  const char *username = NULL;
+  g_autoptr(GPtrArray) services = NULL;
   const char *password = NULL;
   g_autoptr(GError) error = NULL;
 
-  config = goa_dav_client_discover_finish (client, result, &error);
-  if (config == NULL)
+  services = goa_dav_client_discover_finish (client, result, &error);
+  if (services == NULL)
     {
-      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_dialog_report_error (data->dialog, error);
       return;
     }
 
-  if (!add_account_handle_response (task, config))
+  if (!add_account_handle_response (task, services))
     return;
 
-  username = gtk_editable_get_text (GTK_EDITABLE (data->username));
   password = gtk_editable_get_text (GTK_EDITABLE (data->password));
 
   /* WebDAV needs discovery to get the redirected URI for GVfs */
   if (data->check_stage == GOA_PROVIDER_FEATURE_FILES)
     {
       goa_dav_client_discover (client,
-                               data->check_uri,
-                               username,
+                               goa_dav_config_get_uri (data->check_config),
+                               goa_dav_config_get_username (data->check_config),
                                password,
                                data->accept_ssl_errors,
                                cancellable,
@@ -796,8 +828,7 @@ add_account_discover_cb (GoaDavClient *client,
   else
     {
       goa_dav_client_check (client,
-                            data->check_uri,
-                            username,
+                            data->check_config,
                             password,
                             data->accept_ssl_errors,
                             cancellable,
@@ -851,7 +882,6 @@ add_account_action_cb (GoaProviderDialog *dialog,
     }
 
   /* Confirm the account */
-  g_clear_pointer (&data->config, goa_dav_configuration_free);
   dav_client = goa_dav_client_new ();
   goa_dav_client_discover (dav_client,
                            uri,
@@ -868,7 +898,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
 static void
 add_account (GoaProvider         *provider,
              GoaClient           *client,
-             GtkWindow           *parent,
+             GtkWidget           *parent,
              GCancellable        *cancellable,
              GAsyncReadyCallback  callback,
              gpointer             user_data)
@@ -902,12 +932,10 @@ refresh_account_credentials_cb (GoaAccount   *account,
                                 gpointer      user_data)
 {
   g_autoptr(GTask) task = G_TASK (g_steal_pointer (&user_data));
-  AddAccountData *data = g_task_get_task_data (task);
   GError *error = NULL;
 
   if (!goa_account_call_ensure_credentials_finish (account, NULL, res, &error))
     {
-      data->accept_ssl_errors = (error->code == GOA_ERROR_SSL);
       goa_provider_task_return_error (task, error);
       return;
     }
@@ -1015,23 +1043,18 @@ refresh_account_action_cb (GoaProviderDialog *dialog,
 {
   AddAccountData *data = g_task_get_task_data (task);
   GCancellable *cancellable = g_task_get_cancellable (task);
-  const char *uri;
   const char *password;
-  const char *username;
   g_autoptr(GoaDavClient) dav_client = NULL;
 
   if (goa_provider_dialog_get_state (data->dialog) != GOA_DIALOG_BUSY)
     return;
 
-  uri = gtk_editable_get_text (GTK_EDITABLE (data->uri));
-  username = gtk_editable_get_text (GTK_EDITABLE (data->username));
   password = gtk_editable_get_text (GTK_EDITABLE (data->password));
 
   /* Confirm the account */
   dav_client = goa_dav_client_new ();
   goa_dav_client_check (dav_client,
-                        uri,
-                        username,
+                        data->check_config,
                         password,
                         data->accept_ssl_errors,
                         cancellable,
@@ -1045,7 +1068,7 @@ static void
 refresh_account (GoaProvider         *provider,
                  GoaClient           *client,
                  GoaObject           *object,
-                 GtkWindow           *parent,
+                 GtkWidget           *parent,
                  GCancellable        *cancellable,
                  GAsyncReadyCallback  callback,
                  gpointer             user_data)
@@ -1056,7 +1079,7 @@ refresh_account (GoaProvider         *provider,
   g_assert (GOA_IS_WEBDAV_PROVIDER (provider));
   g_assert (GOA_IS_CLIENT (client));
   g_assert (GOA_IS_OBJECT (object));
-  g_assert (parent == NULL || GTK_IS_WINDOW (parent));
+  g_assert (parent == NULL || GTK_IS_WIDGET (parent));
   g_assert (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
   data = g_new0 (AddAccountData, 1);
